unit Rtdbdlg;

{ RTDBComboDlg                                                  }
{ Copyright 1996 R&T Software, Tomasz Stanczak                  }
{ CIS 100735,3273                                               }
{ WWW http://ourworld.compuserve.com/homepages/tomasz           }
{                                                               }
{ TRTComboDlg is a data-aware version of TRTComboDlg, MaskEdit  }
{ descendant with a SpeedButton inside. On pressing this button }
{ a OnDialog event is fired, so that you can show whatever you  }
{ need and set its as you wish. You need also a TRTComboDlg in  }
{ order to use this component                                   }
{                                                               }
{ This component is a freeware, so you're granted an unlimited  }
{ license to use it for anything you want, except that you're   }
{ not allowed to distribute and/or market it as your own work,  }
{ alone or as a part of any component or component libraries in }
{ source code or compiled DCUs.                                 }
{                                                               }
{ All rights not explicitly granted here are reserved by the    }
{ author.                                                       }
{                                                               }
{ There are no warranties implied or explicite. You cannot make }
{ me responsible for anything you do with this component and I  }
{ won't promise to support it.                                  }

interface

uses
	WinTypes, WinProcs, Messages, SysUtils, Classes, Controls,
   DB, DBTables, Rtcbdlg;

type
	TRtDBComboDlg = class(TRtComboDlg)
   private
   	FFocused: Boolean;
   	FDataLink: TFieldDataLink;
		procedure DataChange(Sender: TObject);
		procedure EditingChange(Sender: TObject);
		function GetDataField: string;
		function GetDataSource: TDataSource;
		function GetField: TField;
		function GetReadOnly: Boolean;
		procedure SetDataField(const Value: string);
		procedure SetDataSource(Value: TDataSource);
		procedure SetFocused(Value: Boolean);
		procedure SetReadOnly(Value: Boolean);
		procedure UpdateData(Sender: TObject);
		procedure WMCut(var Message: TMessage); message WM_CUT;
		procedure WMPaste(var Message: TMessage); message WM_PASTE;
		procedure CMEnter(var Message: TCMEnter); message CM_ENTER;
		procedure CMExit(var Message: TCMExit); message CM_EXIT;
	protected
		procedure Change; override;
		function EditCanModify: Boolean; override;
		procedure KeyDown(var Key: Word; Shift: TShiftState); override;
		procedure KeyPress(var Key: Char); override;
		procedure Notification(AComponent: TComponent;
			Operation: TOperation); override;
		procedure Reset; override;
		function CanEdit: Boolean; override;
	public
		constructor Create(AOwner: TComponent); override;
		destructor Destroy; override;
		property Field: TField read GetField;
	published
		property DataField: string read GetDataField write SetDataField;
		property DataSource: TDataSource read GetDataSource write SetDataSource;
		property ReadOnly: Boolean read GetReadOnly write SetReadOnly default False;
   end;

procedure Register;

implementation

procedure Register;
begin
	RegisterComponents('RiT',[TRTDBComboDlg]);
end;

{ TRtDBComboDlg }
constructor TRtDBComboDlg.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);
	inherited ReadOnly := True;
	FDataLink := TFieldDataLink.Create;
	FDataLink.Control := Self;
	FDataLink.OnDataChange := DataChange;
	FDataLink.OnEditingChange := EditingChange;
	FDataLink.OnUpdateData := UpdateData;
end;

destructor TRtDBComboDlg.Destroy;
begin
	FDataLink.OnDataChange := nil;
	FDataLink.OnEditingChange := nil;
	FDataLink.Free;
   FDataLink := nil;
   inherited Destroy;
end;

procedure TRtDBComboDlg.Notification(AComponent: TComponent;
	Operation: TOperation);
begin
	inherited Notification(AComponent, Operation);
	if (Operation = opRemove) and (FDataLink <> nil) and
		(AComponent = DataSource) then DataSource := nil;
end;

procedure TRtDBComboDlg.KeyDown(var Key: Word; Shift: TShiftState);
begin
	inherited KeyDown(Key, Shift);
	if (Key = VK_DELETE) or ((Key = VK_INSERT) and (ssShift in Shift)) then
		FDataLink.Edit;
end;

procedure TRtDBComboDlg.KeyPress(var Key: Char);
begin
	inherited KeyPress(Key);
	if (Key in [#32..#255]) and (FDataLink.Field <> nil) and
		not FDataLink.Field.IsValidChar(Key) then
	begin
		MessageBeep(0);
		Key := #0;
	end;
	case Key of
		^H, ^V, ^X, #32..#255:
      begin
			FDataLink.Edit;
			if not CanEdit then
         	Key := #0;
      end;
		#27:
		begin
      	Reset;
			Key := #0;
      end;
	end;
end;

function TRtDBComboDlg.EditCanModify: Boolean;
begin
	Result := FDataLink.Edit;
end;

procedure TRtDBComboDlg.Reset;
begin
	FDataLink.Reset;
	SelectAll;
end;

function TRtDBComboDlg.CanEdit: Boolean;
begin
	Result := (Style=csEditable) and (not ReadOnly);
end;

procedure TRtDBComboDlg.SetFocused(Value: Boolean);
begin
	if FFocused <> Value then
	begin
		FFocused := Value;
		FDataLink.Reset;
	end;
end;

procedure TRtDBComboDlg.Change;
begin
	if FDataLink.Editing then
		FDataLink.Modified;
	inherited Change;
end;

function TRtDBComboDlg.GetDataSource: TDataSource;
begin
	Result := FDataLink.DataSource;
end;

procedure TRtDBComboDlg.SetDataSource(Value: TDataSource);
begin
	FDataLink.DataSource := Value;
end;

function TRtDBComboDlg.GetDataField: string;
begin
	Result := FDataLink.FieldName;
end;

procedure TRtDBComboDlg.SetDataField(const Value: string);
begin
	FDataLink.FieldName := Value;
end;

function TRtDBComboDlg.GetReadOnly: Boolean;
begin
	Result := FDataLink.ReadOnly;
end;

procedure TRtDBComboDlg.SetReadOnly(Value: Boolean);
begin
	FDataLink.ReadOnly := Value;
end;

function TRtDBComboDlg.GetField: TField;
begin
	Result := FDataLink.Field;
end;

procedure TRtDBComboDlg.DataChange(Sender: TObject);
begin
   if FDataLink.Field<>nil then
   begin
      if Alignment<>FDataLink.Field.Alignment then
      begin
         EditText := '';  {forces update}
         Alignment := FDataLink.Field.Alignment;
      end;
      EditMask := FDataLink.Field.EditMask;
      if FDataLink.Field.DataType = ftString then
         MaxLength := FDataLink.Field.Size
      else
         MaxLength := 0;
      if FFocused and FDataLink.CanModify then
         Text := FDataLink.Field.Text
      else
         EditText := FDataLink.Field.DisplayText;
   end
   else if csDesigning in ComponentState then
      EditText := Name;
end;

procedure TRtDBComboDlg.EditingChange(Sender: TObject);
begin
	inherited ReadOnly := not FDataLink.Editing;
end;

procedure TRtDBComboDlg.UpdateData(Sender: TObject);
begin
	ValidateEdit;
	FDataLink.Field.Text := Text;
end;

procedure TRtDBComboDlg.WMPaste(var Message: TMessage);
begin
	FDataLink.Edit;
	inherited;
end;

procedure TRtDBComboDlg.WMCut(var Message: TMessage);
begin
	FDataLink.Edit;
	inherited;
end;

procedure TRtDBComboDlg.CMEnter(var Message: TCMEnter);
begin
	SetFocused(True);
	inherited;
end;

procedure TRtDBComboDlg.CMExit(var Message: TCMExit);
begin
	try
		FDataLink.UpdateRecord;
	except
		SelectAll;
		SetFocus;
		raise;
	end;
	SetFocused(False);
	SetCursor(0);
	DoExit;
end;

end.
